package tacv.universe.map;


import java.io.File;
import java.util.ArrayList;
import java.util.List;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import tacv.universe.environment.AgentScoutEnvironment;
import tacv.universe.map.xml.Box;
import tacv.universe.map.xml.Map;
import tacv.util.Point;

public class UrbanMap  {

    private static final long serialVersionUID = 7530622762977059100L;

    private StreetGraph streetGraph;
    private List<Building> buildings;


    public UrbanMap(AgentScoutEnvironment environment) {
        streetGraph = new StreetGraph(environment);
        buildings = new ArrayList<Building>();

    }

    public UrbanMap(AgentScoutEnvironment environment, File mapXML)  {
        this(environment);
        loadFromXML(mapXML);
    }

    public MapPosition getRandomMapPoint() {
        return streetGraph.getRandomNode();
    }


    public List<Building> getBuildings() {
        return buildings;
    }

    public StreetGraph getStreetGraph() {
        return streetGraph;
    }


    /**
     * Loads map from the provided XML file.
     *
     * @param mapXML the XML file containing the map of the scenario village
     */
    private void loadFromXML(File mapXML) {

        // Load nodes
        JAXBContext c;
        Unmarshaller u;
        Map map = null;

        try {
            c = JAXBContext.newInstance(Map.class);
            u = c.createUnmarshaller();
            map = (Map) u.unmarshal(mapXML);
        } catch (JAXBException e) {
            throw new RuntimeException("Unable to load map from the XML file. Details follow:\n" + e.getStackTrace());
        }

        streetGraph.loadFromMarshalledMap(map);

        // Load buildings
        for (Box box : map.getMesh().getBoxes().getBox()) {
            ArrayList<Point> points = new ArrayList<Point>();
            for (tacv.universe.map.xml.Point xmlpointbox : box.getPoint()) {
                points.add(new Point(xmlpointbox.getX(), xmlpointbox.getY(), box.getBaseAltitude() ));
            }
            buildings.add(new Building(points, box.getBaseAltitude(), box.getHeight()));
        }
        System.out.println("Loaded: " + buildings.size() + " buildings");
    }

}
