package tacv.universe.map.occlusions;

import java.util.*;
import javax.vecmath.Point2d;
import javax.vecmath.Point3d;
import tacv.universe.map.Building;


public class BuildingMap {

    protected double worldOriginX;
    protected double worldOriginY;
    protected double worldSizeX;
    protected double worldSizeY;
    
    protected List<Building> buildings;
    protected List<Wall> walls;
    
    protected int gridCellCount;
    protected double gridCellSizeX;
    protected double gridCellSizeY;
    
    protected Set<Wall>[][] wallsInGrid;
    protected Set<Building>[][] buildingsInGrid;


    public BuildingMap(int gridCellCount, double worldSizeX, double worldSizeY, List<Building> buildings) {
        this.gridCellCount = gridCellCount;

        worldOriginX = 0;
        worldOriginY = 0;
        this.worldSizeX = worldSizeX;
        this.worldSizeY = worldSizeY;
        
        this.buildings = buildings;
        walls = new ArrayList<Wall>();
        for (Building building : buildings) {
            for (Wall wall : building.getWalls()) {
                walls.add(wall);
            }
        }
        
        gridCellSizeX = worldSizeX / (double) gridCellCount;
        gridCellSizeY = worldSizeY / (double) gridCellCount;

        initWallsInGrid();
        initBuildingsInGrid();
    }

    private void initWallsInGrid() {
        wallsInGrid = new Set[gridCellCount][gridCellCount];
        for (int j = 0; j < gridCellCount; j++) {
            for (int i = 0; i < gridCellCount; i++) {
                wallsInGrid[i][j] = new HashSet<Wall>();
            }
        }
        for (Wall wall : walls) {
            int i0 = (int)((wall.point0.x - worldOriginX) / gridCellSizeX);
            int j0 = (int)((wall.point0.y - worldOriginY) / gridCellSizeY);
            int i1 = (int)((wall.point1.x - worldOriginX) / gridCellSizeX);
            int j1 = (int)((wall.point1.y - worldOriginY) / gridCellSizeY);
            int i2 = (int)((wall.point2.x - worldOriginX) / gridCellSizeX);
            int j2 = (int)((wall.point2.y - worldOriginY) / gridCellSizeY);
            wallsInGrid[i0][j0].add(wall);
            wallsInGrid[i1][j1].add(wall);
            wallsInGrid[i2][j2].add(wall);
        }        
    }

    private void initBuildingsInGrid() {
        buildingsInGrid = new Set[gridCellCount][gridCellCount];
        for (int j = 0; j < gridCellCount; j++) {
            for (int i = 0; i < gridCellCount; i++) {
                Set<Building> set = new HashSet<Building>();
                buildingsInGrid[i][j] = set;
                for (Wall wall : wallsInGrid[i][j]) {
                    set.add(wall.building);
                }
            }
        }
    }

    public Set<Wall> getWalls(Point3d cameraPosition, double cameraRadius) {
        Set<Wall> set = new LinkedHashSet<Wall>();
        int iMin = (int)((cameraPosition.x - cameraRadius - worldOriginX) / gridCellSizeX);
        int iMax = (int)((cameraPosition.x + cameraRadius - worldOriginX) / gridCellSizeX);
        int jMin = (int)((cameraPosition.y - cameraRadius - worldOriginY) / gridCellSizeY);
        int jMax = (int)((cameraPosition.y + cameraRadius - worldOriginY) / gridCellSizeY);
        if (iMin < 0) {
            iMin = 0;
        }
        if (jMin < 0) {
            jMin = 0;
        }
        if (iMax >= gridCellCount) {
            iMax = gridCellCount - 1;
        }
        if (jMax >= gridCellCount) {
            jMax = gridCellCount - 1;
        }
        for (int j = jMin; j <= jMax; j++) {
            for (int i = iMin; i <= iMax; i++) {
                set.addAll(wallsInGrid[i][j]);
            }
        }
        return set;
    }
    
    public static double distance(Point2d p1, Point2d p2) {
        double dx = p1.x - p2.x;
        double dy = p1.y - p2.y;
        return Math.sqrt(dx * dx + dy * dy);
    }
}
