package tacv.vis;

import com.jme3.math.Vector3f;
import tacv.util.Point;
import cz.agents.alite.vis.Vis;
import cz.agents.alite.vis.element.Line;
import cz.agents.alite.vis.element.StyledPoint;
import cz.agents.alite.vis.element.aggregation.LineElements;
import cz.agents.alite.vis.element.aggregation.StyledPointElements;
import cz.agents.alite.vis.element.implemetation.LineImpl;
import cz.agents.alite.vis.element.implemetation.StyledPointImpl;
import cz.agents.alite.vis.layer.GroupLayer;
import cz.agents.alite.vis.layer.VisLayer;
import cz.agents.alite.vis.layer.common.CommonLayer;
import cz.agents.alite.vis.layer.terminal.ArrowLayer;
import cz.agents.alite.vis.layer.terminal.StyledPointLayer;
import java.awt.Color;
import java.awt.Graphics2D;
import java.util.ArrayList;
import javax.vecmath.Point3d;
import tacv.universe.entity.embodiment.AutomaticRegularCar;
import tacv.universe.environment.storage.CarStorage;

public class CarLayer extends CommonLayer {

    public static VisLayer create(final CarStorage carStorage) {
        GroupLayer group = GroupLayer.create();

        group.setHelpOverrideString("[Car] This layer shows car related informatioin (position, heading vector, next waypont).");
        
        group.addSubLayer(StyledPointLayer.create(new StyledPointElements() {

            @Override
            public Iterable<? extends StyledPoint> getPoints() {

                ArrayList<StyledPoint> points = new ArrayList<StyledPoint>();
                for (AutomaticRegularCar carInfo : carStorage.getCars().values()) {

                    Color color = Color.DARK_GRAY;

                    Point waypoint = carInfo.getWaypoint();
                    if (waypoint != null) {
                        points.add(new StyledPointImpl(waypoint, Color.BLACK, 8));
                    }
                    points.add(new StyledPointImpl(carInfo.getActualPosition(), color, 12));

                }
                return points;
            }
        }));

        //draw arrow
        group.addSubLayer(ArrowLayer.create(new LineElements() {

            @Override
            public Color getColor() {
                return Color.BLACK;
            }

            @Override
            public int getStrokeWidth() {
                return 1;
            }

            @Override
            public Iterable<? extends Line> getLines() {
                ArrayList<Line> lines = new ArrayList<Line>();
                for (AutomaticRegularCar carInfo : carStorage.getCars().values()) {
                    Point3d position = carInfo.getActualPosition();
                    Point3d arrowTip = (Point3d) position.clone();
                    Vector3f ho = carInfo.getHeading();
                    Point3d heading = new Point3d(ho.x, ho.y, ho.z);
                    heading.scale(25);
                    arrowTip.add(heading);
                    lines.add(new LineImpl(position, arrowTip));
                }
                return lines;
            }
        }));

        //draw position info
        group.addSubLayer(new CommonLayer() {

            @Override
            public void paint(Graphics2D canvas) {
                super.paint(canvas);
                canvas.setColor(Color.BLACK);
                for (AutomaticRegularCar carInfo : carStorage.getCars().values()) {
                    //draw helicopter position info
                    Point position = carInfo.getActualPosition();
                    String textPosition = "[" + (int) position.x + "; " + (int) position.y + "; " + (int) position.z + "] " + ((int) (carInfo.getSpeed() * 100) / 100f);
                    canvas.drawString(textPosition, (int) (Vis.transX(position.x)), (int) (Vis.transY(position.y)));

                    //draw waypoint info
//            		Point waypoint = helicopterInfo.getWaypoint();
//            		String textWaypoint = "["+(int)waypoint.x + "; " + (int)waypoint.y + "; " +(int)waypoint.z + "]";
//            		canvas.drawString(textWaypoint, (int)waypoint.x, (int)waypoint.y);
                }
            }

            @Override
            public String getLayerDescription() {
                return "Shows car position text.";
            }

            
        });

        return group;
    }
}
